/**************************************************************************************

example1.sas : Regression diagnostics for ORTH

Richard Conrad Zink, Bahjat F. Qaqish & Kunthel By.
2003-2011.

This file provides details and an example for the computing regression diagnostics in 
for the method of Orthogonalized Residuals. Details on  Orthogonalized Residuals 
can be found in 

Zink, R.C. & Qaqish, B.F. (2011). Orthogonalized residuals for estimation of marginally
   specified association parameters in multivariate binary data. Submitted.

The example uses a subset of the 6-City Data discussed in

Ware, J.H., Dockery, D.W., Spiro, A., Speizer, F.E. & Ferris, B.G. (1984). Passive smoking,  
    gas cooking, and respiratory health of children living in six cities. Am. Rev. Respir.  
    Dis. 129, 366-74.  

********************************* DATA ************************************

   y1: Respiratory illness at age 7 (0 = no, 1 = yes)  
   y2: Respiratory illness at age 8 (0 = no, 1 = yes)
   y3: Respiratory illness at age 9 (0 = no, 1 = yes)
   y4: Respiratory illness at age 10 (0 = no, 1 = yes)
smoke: Mothers Smoking Status at Baseline (0 = not, 1 = smoker)
count: Frequency of cluster

Full Model for mean will have an intercept, agecent (age centered around 9) and smoke.
Full Model for OR will have an intercept, agediff (|difference in ages for (j,k)th visits|)  
and smoke.

***************************** MACRO INPUTS ********************************

 xydata: The data set containing the outcome and marginal mean covariates  
   yvar: The 0/1 binary outcome in xydata
   xvar: A list of marginal mean covariates in xydata. Must all be numeric
     id: Cluster ID (consecutive integers) in xydata
  zdata: The data set containing the covariates for the marginal OR regression
   zvar: A list of marginal OR covariates in zdata. Must all be numeric  
  wdata: The data set containing cluster weights
   wvar: The weight variable in wdata (numeric, i.e. cluster frequencies)
maxiter: Maximum number of iterations for Fisher Scoring
epsilon: Tolerance for convergence
estlamb: Lambda fixed at 0 (no), or estimate (yes)

****************************** COMMENTS ***********************************

1. All inputs to the macro ORTHRES are REQUIRED. Data is assumed to be non-missing.
   The software may compile with missing data, but we can not be sure of the final
   results. Either delete individual observations with missing data or impute values
   for completeness.

2. IDs should go from 1 to K, where K is the number of clusters. Data should be sorted
   by ID before calling the macro. This goes for all data sets: W, Z, XY.
    
3. Each row of XYdata represents 1 observation. If n_i is the sample size for the ith  
   cluster, the data appears as
       
      Observation 1, Cluster 1
      Observation 2, Cluster 1
      ...
      Observation n_i, Cluster 1
      Observation 1, Cluster 2
      Observation 2, Cluster 2
      ...
      Observation n_K, Cluster K
    
4. Wdata should have K observations, 1 for each cluster. This represents the frequency
   of a particular cluster within the data. Making use of Wdata can greatly increase the
   speed of the ORTHRES. If all clusters appear only once, the weights should be 1.
      
      Cluster 1, Weight 1
      Cluster 2, Weight 2
      ...
      Cluster K, Weight K
    
5. Zdata contains one observation for all pairs (j,k) where j < k for all clusters i.  
   Say cluster 1 has 5 obs and cluster K has 4 obs, then the first 10 obs and last 6 obs 
   of Zdata should be for the pairs  
   
      Cluster 1, (1,2) Covariates 
      Cluster 1, (1,3) Covariates 
      Cluster 1, (1,4) Covariates 
      Cluster 1, (1,5) Covariates 
      Cluster 1, (2,3) Covariates 
      Cluster 1, (2,4) Covariates 
      Cluster 1, (2,5) Covariates 
      Cluster 1, (3,4) Covariates 
      Cluster 1, (3,5) Covariates 
      Cluster 1, (4,5) Covariates 
      ...
      Cluster K, (1,2) Covariates 
      Cluster K, (1,3) Covariates 
      Cluster K, (1,4) Covariates 
      Cluster K, (2,3) Covariates 
      Cluster K, (2,4) Covariates 
      Cluster K, (3,4) Covariates 

   IMPORTANT! Clusters of size 1 SHOULD NOT have rows within the Zdata data set. For  
   example, say there are 3 clusters of size 5, 1, 4. Zdata will have 16 rows, the first  
   10 rows for cluster 1, and the last 6 will be for cluster 3.
    
6. The value for lambda is assumed to be within range if -1/(max(n)*(max(n)-1)/2 - 1) <
   lambda < 1. If lambda goes out of range, a warning is printed and it is reset to 0.8
   times the limit that is exceeded.

7. The code checks for positive-definiteness of var(Y_i) for each cluster when the robust  
   covariance matrix is computed upon convergence. If var(Y_i) is non-PD, this is an  
   indication that a joint distribution does not exist for the particular cluster with  
   the given parameter estimates. However, this is a necessary but not sufficient  
   condition for the existence of a joint distribution. This is the best that can be done  
   presently. A warning will be printed if this occurs.
    
**************************************************************************************/

options ls=72; * mprint mlogic;


%include "orthrescls.macro";

/* Read in the 6-City Data */

data city;
   input y1 y2 y3 y4 smoke count;
   cards;

   0   0   0   0   0   237
   0   0   0   1   0    10
   0   0   1   0   0    15
   0   0   1   1   0     4
   0   1   0   0   0    16
   0   1   0   1   0     2
   0   1   1   0   0     7
   0   1   1   1   0     3
   1   0   0   0   0    24
   1   0   0   1   0     3
   1   0   1   0   0     3
   1   0   1   1   0     2
   1   1   0   0   0     6
   1   1   0   1   0     2
   1   1   1   0   0     5
   1   1   1   1   0    11
   0   0   0   0   1   118
   0   0   0   1   1     6
   0   0   1   0   1     8
   0   0   1   1   1     2
   0   1   0   0   1    11
   0   1   0   1   1     1
   0   1   1   0   1     6
   0   1   1   1   1     4
   1   0   0   0   1     7
   1   0   0   1   1     3
   1   0   1   0   1     3
   1   0   1   1   1     1
   1   1   0   0   1     4
   1   1   0   1   1     2
   1   1   1   0   1     4
   1   1   1   1   1     7
   ;

data xy(keep = id y int agecent smoke) zz(keep = id int agediff smoke) ww(keep = id count);
   set city;
   retain id 1;
      /* Create XY data. agecent is age centered around 9 */
      y=y1; int=1; agecent=-2; output xy;
      y=y2; int=1; agecent=-1; output xy;
      y=y3; int=1; agecent=0; output xy;
      y=y4; int=1; agecent=1; output xy;
       
      /* Create W data */
      output ww;
       
      /* Create Z data. agediff is |difference in ages for (j,k)th visits| */
      int=1; agediff=1; output zz;
      int=1; agediff=2; output zz;
      int=1; agediff=3; output zz;
      int=1; agediff=1; output zz;
      int=1; agediff=2; output zz;
      int=1; agediff=1; output zz;
      id + 1;
      run;



/* This call of ORTHRES estimates Lambda. Exchangeable Model for Mean and OR. */

%ORTHRESCLS(xydata=xy, yvar=y, xvar=int agecent smoke, id=id, zdata=zz, zvar=int,  
            wdata=ww, wvar=count, maxiter=20, epsilon=1e-5, estlamb = YES ,
            CLSOUT=clsout, OBSOUT=obsout, monitor=no);

proc print data = clsout; run;
proc print data = obsout; run; 

